@@ -11,16 +11,19 @@
 #define PACKAGE_NAME "Chocolate Doom"
 
 /* Define to the full name and version of this package. */
-#define PACKAGE_STRING "Chocolate Doom 1.4.0"
+#define PACKAGE_STRING "Chocolate Doom 1.6.0"
 
 /* Define to the one symbol short name of this package. */
 #define PACKAGE_TARNAME "chocolate-doom"
 
 /* Define to the version of this package. */
-#define PACKAGE_VERSION "1.4.0"
+#define PACKAGE_VERSION "1.6.0"
+
+/* Change this when you create your awesome forked version */
+#define PROGRAM_PREFIX "chocolate-"
 
 /* Version number of package */
-#define VERSION "1.4.0"
+#define VERSION "1.6.0"
 
 /* Define to 1 if your processor stores words with the most significant byte
    first (like Motorola and SPARC, unlike Intel and VAX). */
