@@ -124,8 +124,6 @@ int		startmap;
 boolean		autostart;
 int             startloadgame;
 
-FILE*		debugfile;
-
 boolean		advancedemo;
 
 // Store demo, do not accept any inputs
@@ -399,14 +397,6 @@ void D_DoomLoop (void)
     if (demorecording)
 	G_BeginRecording ();
 		
-    if (M_CheckParm ("-debugfile"))
-    {
-	char    filename[20];
-	sprintf (filename,"debug%i.txt",consoleplayer);
-	printf ("debug output to: %s\n",filename);
-	debugfile = fopen (filename,"w");
-    }
-
     TryRunTics();
 
     I_InitGraphics ();
@@ -504,7 +494,16 @@ void D_DoAdvanceDemo (void)
     paused = false;
     gameaction = ga_nothing;
 
-    if (gamemode == retail && gameversion != exe_chex)
+    // The Ultimate Doom executable changed the demo sequence to add
+    // a DEMO4 demo.  Final Doom was based on Ultimate, so also
+    // includes this change; however, the Final Doom IWADs do not
+    // include a DEMO4 lump, so the game bombs out with an error
+    // when it reaches this point in the demo sequence.
+
+    // However! There is an alternate version of Final Doom that
+    // includes a fixed executable.
+
+    if (gameversion == exe_ultimate || gameversion == exe_final)
       demosequence = (demosequence+1)%7;
     else
       demosequence = (demosequence+1)%6;
@@ -660,8 +659,10 @@ static struct
     GameVersion_t version;
 } gameversions[] = {
     {"Doom 1.9",             "1.9",        exe_doom_1_9},
+    {"Hacx",                 "hacx",       exe_hacx},
     {"Ultimate Doom",        "ultimate",   exe_ultimate},
     {"Final Doom",           "final",      exe_final},
+    {"Final Doom (alt)",     "final2",     exe_final2},
     {"Chex Quest",           "chex",       exe_chex},
     { NULL,                  NULL,         0},
 };
@@ -678,12 +679,12 @@ static void InitGameVersion(void)
     // @category compat
     //
     // Emulate a specific version of Doom.  Valid values are "1.9",
-    // "ultimate" and "final".
+    // "ultimate", "final", "final2", "hacx" and "chex".
     //
 
-    p = M_CheckParm("-gameversion");
+    p = M_CheckParmWithArgs("-gameversion", 1);
 
-    if (p > 0)
+    if (p)
     {
         for (i=0; gameversions[i].description != NULL; ++i)
         {
@@ -711,7 +712,7 @@ static void InitGameVersion(void)
     {
         // Determine automatically
 
-        if (gameversion == exe_chex) 
+        if (gameversion == exe_chex || gameversion == exe_hacx)
         {
             // Already determined
         }
@@ -734,8 +735,10 @@ static void InitGameVersion(void)
             else
             {
                 // Final Doom: tnt or plutonia
+                // Default to the "alt" version of the executable that
+                // fixes the demo loop behavior.
 
-                gameversion = exe_final;
+                gameversion = exe_final2;
             }
         }
     }
@@ -774,11 +777,37 @@ void PrintGameVersion(void)
 
 static void LoadChexDeh(void)
 {
-    char *chex_deh;
+    char *chex_deh = NULL;
+    char *sep;
 
     if (gameversion == exe_chex)
     {
-        chex_deh = D_FindWADByName("chex.deh");
+        // Look for chex.deh in the same directory as the IWAD file.
+
+        sep = strrchr(iwadfile, DIR_SEPARATOR);
+
+        if (sep != NULL)
+        {
+            chex_deh = malloc(strlen(iwadfile) + 9);
+            strcpy(chex_deh, iwadfile);
+            chex_deh[sep - iwadfile + 1] = '\0';
+            strcat(chex_deh, "chex.deh");
+        }
+        else
+        {
+            chex_deh = strdup("chex.deh");
+        }
+
+        // If the dehacked patch isn't found, try searching the WAD
+        // search path instead.  We might find it...
+
+        if (!M_FileExists(chex_deh))
+        {
+            free(chex_deh);
+            chex_deh = D_FindWADByName("chex.deh");
+        }
+
+        // Still not found?
 
         if (chex_deh == NULL)
         {
@@ -796,6 +825,20 @@ static void LoadChexDeh(void)
     }
 }
 
+static void LoadHacxDeh(void)
+{
+    // If this is the HACX IWAD, we need to load the DEHACKED lump.
+
+    if (gameversion == exe_hacx)
+    {
+        if (!DEH_LoadLumpByName("DEHACKED"))
+        {
+            I_Error("DEHACKED lump not found.  Please check that this is the "
+                    "Hacx v1.2 IWAD.");
+        }
+    }
+}
+
 //
 // D_DoomMain
 //
@@ -838,6 +881,21 @@ void D_DoomMain (void)
         // Never returns
     }
 
+    //!
+    // @category net
+    //
+    // Query the Internet master server for a global list of active
+    // servers.
+    //
+
+    if (M_CheckParm("-search"))
+    {
+        printf("\nSearching for servers on Internet ...\n");
+        p = NET_MasterQuery(NET_QueryPrintCallback, NULL);
+        printf("\n%i server(s) found.\n", p);
+        exit(0);
+    }
+
     //!
     // @arg <address>
     // @category net
@@ -846,11 +904,12 @@ void D_DoomMain (void)
     // address.
     //
 
-    p = M_CheckParm("-query");
+    p = M_CheckParmWithArgs("-query", 1);
 
-    if (p > 0)
+    if (p)
     {
         NET_QueryAddress(myargv[p+1]);
+        exit(0);
     }
 
     //!
@@ -859,8 +918,13 @@ void D_DoomMain (void)
     // Search the local LAN for running servers.
     //
 
-    if (M_CheckParm("-search"))
-        NET_LANQuery();
+    if (M_CheckParm("-localsearch"))
+    {
+        printf("\nSearching for servers on local LAN ...\n");
+        p = NET_LANQuery(NET_QueryPrintCallback, NULL);
+        printf("\n%i server(s) found.\n", p);
+        exit(0);
+    }
 
 #endif
             
@@ -993,7 +1057,7 @@ void D_DoomMain (void)
     // into the main IWAD.  Multiple files may be specified.
     //
 
-    p = M_CheckParm("-merge");
+    p = M_CheckParmWithArgs("-merge", 1);
 
     if (p > 0)
     {
@@ -1019,7 +1083,7 @@ void D_DoomMain (void)
     // Simulates the behavior of NWT's -merge option.  Multiple files
     // may be specified.
 
-    p = M_CheckParm("-nwtmerge");
+    p = M_CheckParmWithArgs("-nwtmerge", 1);
 
     if (p > 0)
     {
@@ -1044,7 +1108,7 @@ void D_DoomMain (void)
     // the main IWAD directory.  Multiple files may be specified.
     //
 
-    p = M_CheckParm("-af");
+    p = M_CheckParmWithArgs("-af", 1);
 
     if (p > 0)
     {
@@ -1067,7 +1131,7 @@ void D_DoomMain (void)
     // into the main IWAD directory.  Multiple files may be specified.
     //
 
-    p = M_CheckParm("-as");
+    p = M_CheckParmWithArgs("-as", 1);
 
     if (p > 0)
     {
@@ -1089,7 +1153,7 @@ void D_DoomMain (void)
     // Equivalent to "-af <files> -as <files>".
     //
 
-    p = M_CheckParm("-aa");
+    p = M_CheckParmWithArgs("-aa", 1);
 
     if (p > 0)
     {
@@ -1113,7 +1177,7 @@ void D_DoomMain (void)
     // Load the specified PWAD files.
     //
 
-    p = M_CheckParm ("-file");
+    p = M_CheckParmWithArgs("-file", 1);
     if (p)
     {
 	// the parms after p are wadfile/lump names,
@@ -1132,40 +1196,6 @@ void D_DoomMain (void)
     // Debug:
 //    W_PrintDirectory();
 
-    // add any files specified on the command line with -file wadfile
-    // to the wad list
-    //
-    // convenience hack to allow -wart e m to add a wad file
-    // prepend a tilde to the filename so wadfile will be reloadable
-    p = M_CheckParm ("-wart");
-    if (p)
-    {
-	myargv[p][4] = 'p';     // big hack, change to -warp
-
-	// Map name handling.
-	switch (gamemode )
-	{
-	  case shareware:
-	  case retail:
-	  case registered:
-	    sprintf (file,"~"DEVMAPS"E%cM%c.wad",
-		     myargv[p+1][0], myargv[p+2][0]);
-	    printf("Warping to Episode %s, Map %s.\n",
-		   myargv[p+1],myargv[p+2]);
-	    break;
-	    
-	  case commercial:
-	  default:
-	    p = atoi (myargv[p+1]);
-	    if (p<10)
-	      sprintf (file,"~"DEVMAPS"cdata/map0%i.wad", p);
-	    else
-	      sprintf (file,"~"DEVMAPS"cdata/map%i.wad", p);
-	    break;
-	}
-	D_AddFile (file);
-    }
-
     //!
     // @arg <demo>
     // @category demo
@@ -1174,7 +1204,7 @@ void D_DoomMain (void)
     // Play back the demo named demo.lmp.
     //
 
-    p = M_CheckParm ("-playdemo");
+    p = M_CheckParmWithArgs ("-playdemo", 1);
 
     if (!p)
     {
@@ -1186,11 +1216,11 @@ void D_DoomMain (void)
         // Play back the demo named demo.lmp, determining the framerate
         // of the screen.
         //
-	p = M_CheckParm ("-timedemo");
+	p = M_CheckParmWithArgs("-timedemo", 1);
 
     }
 
-    if (p && p < myargc-1)
+    if (p)
     {
         if (!strcasecmp(myargv[p+1] + strlen(myargv[p+1]) - 4, ".lmp"))
         {
@@ -1227,6 +1257,7 @@ void D_DoomMain (void)
     D_IdentifyVersion();
     InitGameVersion();
     LoadChexDeh();
+    LoadHacxDeh();
     D_SetGameDescription();
     D_SetSaveGameDir();
 
@@ -1269,9 +1300,9 @@ void D_DoomMain (void)
     // 0 disables all monsters.
     //
 
-    p = M_CheckParm ("-skill");
+    p = M_CheckParmWithArgs("-skill", 1);
 
-    if (p && p < myargc-1)
+    if (p)
     {
 	startskill = myargv[p+1][0]-'1';
 	autostart = true;
@@ -1284,9 +1315,9 @@ void D_DoomMain (void)
     // Start playing on episode n (1-4)
     //
 
-    p = M_CheckParm ("-episode");
+    p = M_CheckParmWithArgs("-episode", 1);
 
-    if (p && p < myargc-1)
+    if (p)
     {
 	startepisode = myargv[p+1][0]-'0';
 	startmap = 1;
@@ -1303,12 +1334,11 @@ void D_DoomMain (void)
     // For multiplayer games: exit each level after n minutes.
     //
 
-    p = M_CheckParm ("-timer");
+    p = M_CheckParmWithArgs("-timer", 1);
 
-    if (p && p < myargc-1 && deathmatch)
+    if (p)
     {
 	timelimit = atoi(myargv[p+1]);
-	printf("timer: %i\n", timelimit);
     }
 
     //!
@@ -1320,10 +1350,8 @@ void D_DoomMain (void)
 
     p = M_CheckParm ("-avg");
 
-    if (p && p < myargc-1 && deathmatch)
+    if (p)
     {
-        DEH_printf("Austin Virtual Gaming: Levels will end "
-                       "after 20 minutes\n");
 	timelimit = 20;
     }
 
@@ -1335,9 +1363,9 @@ void D_DoomMain (void)
     // (Doom 2)
     //
 
-    p = M_CheckParm ("-warp");
+    p = M_CheckParmWithArgs("-warp", 1);
 
-    if (p && p < myargc-1)
+    if (p)
     {
         if (gamemode == commercial)
             startmap = atoi (myargv[p+1]);
@@ -1381,9 +1409,9 @@ void D_DoomMain (void)
     // Load the game in slot s.
     //
 
-    p = M_CheckParm ("-loadgame");
+    p = M_CheckParmWithArgs("-loadgame", 1);
     
-    if (p && p < myargc-1)
+    if (p)
     {
         startloadgame = atoi(myargv[p+1]);
     }
@@ -1483,24 +1511,24 @@ void D_DoomMain (void)
     // Record a demo named x.lmp.
     //
 
-    p = M_CheckParm ("-record");
+    p = M_CheckParmWithArgs("-record", 1);
 
-    if (p && p < myargc-1)
+    if (p)
     {
 	G_RecordDemo (myargv[p+1]);
 	autostart = true;
     }
 
-    p = M_CheckParm ("-playdemo");
-    if (p && p < myargc-1)
+    p = M_CheckParmWithArgs("-playdemo", 1);
+    if (p)
     {
 	singledemo = true;              // quit after one demo
 	G_DeferedPlayDemo (demolumpname);
 	D_DoomLoop ();  // never returns
     }
 	
-    p = M_CheckParm ("-timedemo");
-    if (p && p < myargc-1)
+    p = M_CheckParmWithArgs("-timedemo", 1);
+    if (p)
     {
 	G_TimeDemo (demolumpname);
 	D_DoomLoop ();  // never returns
