@@ -139,7 +139,7 @@ int             gametic;
 int             levelstarttic;          // gametic at level start 
 int             totalkills, totalitems, totalsecret;    // for intermission 
  
-char            demoname[32]; 
+char           *demoname;
 boolean         demorecording; 
 boolean         longtics;               // cph's doom 1.91 longtics hack
 boolean         lowres_turn;            // low resolution turning for longtics
@@ -182,9 +182,12 @@ int             key_weapon5 = '5';
 int             key_weapon6 = '6';
 int             key_weapon7 = '7';
 int             key_weapon8 = '8';
+int             key_prevweapon = 0;
+int             key_nextweapon = 0;
 
 int             key_pause = KEY_PAUSE;
 int             key_demo_quit = 'q';
+int             key_spy = KEY_F12;
  
 int             mousebfire = 0;
 int             mousebstrafe = 1;
@@ -195,6 +198,9 @@ int             mousebstraferight = -1;
 int             mousebbackward = -1;
 int             mousebuse = -1;
 
+int             mousebprevweapon = -1;
+int             mousebnextweapon = -1;
+
 // Control whether if a mouse button is double clicked, it acts like 
 // "use" has been pressed
 
@@ -207,6 +213,9 @@ int             joybspeed = 2;
 int             joybstrafeleft = -1;
 int             joybstraferight = -1;
 
+int             joybprevweapon = -1;
+int             joybnextweapon = -1;
+
 // fraggle: Disallow mouse and joystick movement to cause forward/backward
 // motion.  Specified with the '-novert' command line parameter.
 // This is an int to allow saving to config file
@@ -234,14 +243,37 @@ static int *weapon_keys[] = {
     &key_weapon8
 };
 
+// Set to -1 or +1 to switch to the previous or next weapon.
+
+static int next_weapon = 0;
+
+// Used for prev/next weapon keys.
+
+static const struct
+{
+    weapontype_t weapon;
+    weapontype_t weapon_num;
+} weapon_order_table[] = {
+    { wp_fist,            wp_fist },
+    { wp_chainsaw,        wp_fist },
+    { wp_pistol,          wp_pistol },
+    { wp_shotgun,         wp_shotgun },
+    { wp_supershotgun,    wp_shotgun },
+    { wp_chaingun,        wp_chaingun },
+    { wp_missile,         wp_missile },
+    { wp_plasma,          wp_plasma },
+    { wp_bfg,             wp_bfg }
+};
+
 #define SLOWTURNTICS	6 
  
 #define NUMKEYS		256 
+#define MAX_JOY_BUTTONS 20
 
 static boolean  gamekeydown[NUMKEYS]; 
 static int      turnheld;		// for accelerative turning 
  
-static boolean  mousearray[4]; 
+static boolean  mousearray[MAX_MOUSE_BUTTONS + 1];
 static boolean *mousebuttons = &mousearray[1];  // allow [-1]
 
 // mouse values are used once 
@@ -255,8 +287,6 @@ static int      dclicktime2;
 static boolean  dclickstate2;
 static int      dclicks2;
 
-#define MAX_JOY_BUTTONS 20
-
 // joystick values are repeated 
 static int      joyxmove;
 static int      joyymove;
@@ -395,7 +425,70 @@ int G_CmdChecksum (ticcmd_t* cmd)
 		 
     return sum; 
 } 
- 
+
+static boolean WeaponSelectable(weapontype_t weapon)
+{
+    // Can't select the super shotgun in Doom 1.
+
+    if (weapon == wp_supershotgun && gamemission == doom)
+    {
+        return false;
+    }
+
+    // Can't select a weapon if we don't own it.
+
+    if (!players[consoleplayer].weaponowned[weapon])
+    {
+        return false;
+    }
+
+    // Can't select the fist if we have the chainsaw, unless
+    // we also have the berserk pack.
+
+    if (weapon == wp_fist
+     && players[consoleplayer].weaponowned[wp_chainsaw]
+     && !players[consoleplayer].powers[pw_strength])
+    {
+        return false;
+    }
+
+    return true;
+}
+
+static int G_NextWeapon(int direction)
+{
+    weapontype_t weapon;
+    int i;
+
+    // Find index in the table.
+
+    if (players[consoleplayer].pendingweapon == wp_nochange)
+    {
+        weapon = players[consoleplayer].readyweapon;
+    }
+    else
+    {
+        weapon = players[consoleplayer].pendingweapon;
+    }
+
+    for (i=0; i<arrlen(weapon_order_table); ++i)
+    {
+        if (weapon_order_table[i].weapon == weapon)
+        {
+            break;
+        }
+    }
+
+    // Switch weapon.
+
+    do
+    {
+        i += direction;
+        i = (i + arrlen(weapon_order_table)) % arrlen(weapon_order_table);
+    } while (!WeaponSelectable(weapon_order_table[i].weapon));
+
+    return weapon_order_table[i].weapon_num;
+}
 
 //
 // G_BuildTiccmd
@@ -523,20 +616,34 @@ void G_BuildTiccmd (ticcmd_t* cmd)
 	dclicks = 0;                   
     } 
 
-    // chainsaw overrides 
+    // If the previous or next weapon button is pressed, the
+    // next_weapon variable is set to change weapons when
+    // we generate a ticcmd.  Choose a new weapon.
 
-    for (i=0; i<arrlen(weapon_keys); ++i)
+    if (next_weapon != 0)
     {
-        int key = *weapon_keys[i];
+        i = G_NextWeapon(next_weapon);
+        cmd->buttons |= BT_CHANGE;
+        cmd->buttons |= i << BT_WEAPONSHIFT;
+        next_weapon = 0;
+    }
+    else
+    {
+        // Check weapon keys.
 
-        if (gamekeydown[key])
+        for (i=0; i<arrlen(weapon_keys); ++i)
         {
-	    cmd->buttons |= BT_CHANGE; 
-	    cmd->buttons |= i<<BT_WEAPONSHIFT; 
-	    break; 
+            int key = *weapon_keys[i];
+
+            if (gamekeydown[key])
+            {
+                cmd->buttons |= BT_CHANGE;
+                cmd->buttons |= i<<BT_WEAPONSHIFT;
+                break;
+            }
         }
     }
-    
+
     // mouse
     if (mousebuttons[mousebforward]) 
     {
@@ -720,7 +827,6 @@ void G_DoLoadLevel (void)
         players[consoleplayer].message = "Press escape to quit.";
     }
 } 
- 
 
 static void SetJoyButtons(unsigned int buttons_mask)
 {
@@ -728,10 +834,54 @@ static void SetJoyButtons(unsigned int buttons_mask)
 
     for (i=0; i<MAX_JOY_BUTTONS; ++i)
     {
-        joybuttons[i] = (buttons_mask & (1 << i)) != 0;
+        int button_on = (buttons_mask & (1 << i)) != 0;
+
+        // Detect button press:
+
+        if (!joybuttons[i] && button_on)
+        {
+            // Weapon cycling:
+
+            if (i == joybprevweapon)
+            {
+                next_weapon = -1;
+            }
+            else if (i == joybnextweapon)
+            {
+                next_weapon = 1;
+            }
+        }
+
+        joybuttons[i] = button_on;
     }
 }
- 
+
+static void SetMouseButtons(unsigned int buttons_mask)
+{
+    int i;
+
+    for (i=0; i<MAX_MOUSE_BUTTONS; ++i)
+    {
+        unsigned int button_on = (buttons_mask & (1 << i)) != 0;
+
+        // Detect button press:
+
+        if (!mousebuttons[i] && button_on)
+        {
+            if (i == mousebprevweapon)
+            {
+                next_weapon = -1;
+            }
+            else if (i == mousebnextweapon)
+            {
+                next_weapon = 1;
+            }
+        }
+
+	mousebuttons[i] = button_on;
+    }
+}
+
 //
 // G_Responder  
 // Get info needed to make ticcmd_ts for the players.
@@ -740,7 +890,7 @@ boolean G_Responder (event_t* ev)
 { 
     // allow spy mode changes even during the demo
     if (gamestate == GS_LEVEL && ev->type == ev_keydown 
-     && ev->data1 == KEY_F12 && (singledemo || !deathmatch) )
+     && ev->data1 == key_spy && (singledemo || !deathmatch) )
     {
 	// spy mode 
 	do 
@@ -800,6 +950,18 @@ boolean G_Responder (event_t* ev)
         testcontrols_mousespeed = abs(ev->data2);
     }
 
+    // If the next/previous weapon keys are pressed, set the next_weapon
+    // variable to change weapons when the next ticcmd is generated.
+
+    if (ev->type == ev_keydown && ev->data1 == key_prevweapon)
+    {
+        next_weapon = -1;
+    }
+    else if (ev->type == ev_keydown && ev->data1 == key_nextweapon)
+    {
+        next_weapon = 1;
+    }
+
     switch (ev->type) 
     { 
       case ev_keydown: 
@@ -820,9 +982,7 @@ boolean G_Responder (event_t* ev)
 	return false;   // always let key up events filter down 
 		 
       case ev_mouse: 
-	mousebuttons[0] = ev->data1 & 1; 
-	mousebuttons[1] = ev->data1 & 2; 
-	mousebuttons[2] = ev->data1 & 4; 
+        SetMouseButtons(ev->data1);
 	mousex = ev->data2*(mouseSensitivity+5)/10; 
 	mousey = ev->data3*(mouseSensitivity+5)/10; 
 	return true;    // eat events 
@@ -1897,14 +2057,14 @@ void G_WriteDemoTiccmd (ticcmd_t* cmd)
 //
 // G_RecordDemo 
 // 
-void G_RecordDemo (char* name) 
+void G_RecordDemo (char *name) 
 { 
     int             i; 
     int				maxsize;
 	
     usergame = false; 
-    strcpy (demoname, name); 
-    strcat (demoname, ".lmp"); 
+    demoname = Z_Malloc(strlen(name) + 5, PU_STATIC, NULL);
+    sprintf(demoname, "%s.lmp", name);
     maxsize = 0x20000;
 
     //!
@@ -1915,8 +2075,8 @@ void G_RecordDemo (char* name)
     // Specify the demo buffer size (KiB)
     //
 
-    i = M_CheckParm ("-maxdemo");
-    if (i && i<myargc-1)
+    i = M_CheckParmWithArgs("-maxdemo", 1);
+    if (i)
 	maxsize = atoi(myargv[i+1])*1024;
     demobuffer = Z_Malloc (maxsize,PU_STATIC,NULL); 
     demoend = demobuffer + maxsize;
@@ -2064,16 +2224,11 @@ void G_DoPlayDemo (void)
     for (i=0 ; i<MAXPLAYERS ; i++) 
 	playeringame[i] = *demo_p++; 
 
-    //!
-    // @category demo
-    // 
-    // Play back a demo recorded in a netgame with a single player.
-    //
-
-    if (playeringame[1] || M_CheckParm("-netdemo") > 0) 
-    { 
-	netgame = true; 
-	netdemo = true; 
+    if (playeringame[1] || M_CheckParm("-solo-net") > 0
+                        || M_CheckParm("-netdemo") > 0)
+    {
+	netgame = true;
+	netdemo = true;
     }
 
     // don't spend a lot of time in loadlevel 
