@@ -30,6 +30,7 @@
 #include "doomtype.h"
 
 #include "i_video.h"
+#include "m_argv.h"
 #include "z_zone.h"
 
 #if defined(_MSC_VER) && !defined(__cplusplus)
@@ -53,11 +54,11 @@ static int dest_pitch;
 // stretch_tables[1] : 40% / 60%
 // All other combinations can be reached from these two tables.
 
-static byte *stretch_tables[2];
+static byte *stretch_tables[2] = { NULL, NULL };
 
 // 50%/50% stretch table, for 800x600 squash mode
 
-static byte *half_stretch_table;
+static byte *half_stretch_table = NULL;
 
 // Called to set the source and destination buffers before doing the
 // scale.
@@ -364,6 +365,11 @@ static byte *GenerateStretchTable(byte *palette, int pct)
 
 static void I_InitStretchTables(byte *palette)
 {
+    if (stretch_tables[0] != NULL)
+    {
+        return;
+    }
+
     // We only actually need two lookup tables:
     //
     // mix 0%   =  just write line 1
@@ -385,6 +391,11 @@ static void I_InitStretchTables(byte *palette)
 
 static void I_InitSquashTable(byte *palette)
 {
+    if (half_stretch_table != NULL)
+    {
+        return;
+    }
+
     printf("I_InitSquashTable: Generating lookup table..");
     fflush(stdout);
     half_stretch_table = GenerateStretchTable(palette, 50);
@@ -954,6 +965,21 @@ static boolean I_Stretch5x(int x1, int y1, int x2, int y2)
         screenp += dest_pitch; bufp += SCREENWIDTH;
     }
 
+    // test hack for Porsche Monty... scan line simulation:
+    // See here: http://www.doomworld.com/vb/post/962612
+
+    if (M_CheckParm("-scanline") > 0)
+    {
+        screenp = (byte *) dest_buffer + 2 * dest_pitch;
+
+        for (y=0; y<1198; y += 3)
+        {
+            memset(screenp, 0, 1600);
+
+            screenp += dest_pitch * 3;
+        }
+    }
+
     return true;
 }
 
