@@ -284,7 +284,7 @@ void NET_WaitForStart(void)
 
         if (!net_client_connected)
         {
-            I_Error("Disconnected from server");
+            I_Error("Lost connection to server");
         }
 
         TXT_Sleep(100);
